@@ -34,11 +34,19 @@ G_BEGIN_DECLS
 #define EV_IS_DOCUMENT_MODEL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EV_TYPE_DOCUMENT_MODEL))
 
 typedef enum {
-	EV_SIZING_BEST_FIT,
+        EV_SIZING_FIT_PAGE,
+	EV_SIZING_BEST_FIT = EV_SIZING_FIT_PAGE, /* Deprecated */
 	EV_SIZING_FIT_WIDTH,
 	EV_SIZING_FREE,
+        EV_SIZING_AUTOMATIC,
 } EvSizingMode;
 
+typedef enum {
+	EV_PAGE_LAYOUT_SINGLE,
+	EV_PAGE_LAYOUT_DUAL,
+	EV_PAGE_LAYOUT_AUTOMATIC,
+} EvPageLayout;
+
 typedef struct _EvDocumentModel        EvDocumentModel;
 typedef struct _EvDocumentModelClass   EvDocumentModelClass;
 
@@ -66,6 +74,9 @@ gdouble          ev_document_model_get_min_scale     (EvDocumentModel *model);
 void             ev_document_model_set_sizing_mode   (EvDocumentModel *model,
 						      EvSizingMode     mode);
 EvSizingMode     ev_document_model_get_sizing_mode   (EvDocumentModel *model);
+void             ev_document_model_set_page_layout   (EvDocumentModel *model,
+						     EvPageLayout     layout);
+EvPageLayout	 ev_document_model_get_page_layout   (EvDocumentModel *model);
 void             ev_document_model_set_rotation      (EvDocumentModel *model,
 						      gint             rotation);
 gint             ev_document_model_get_rotation      (EvDocumentModel *model);
@@ -75,9 +86,6 @@ gboolean       ev_document_model_get_inverted_colors (EvDocumentModel *model);
 void             ev_document_model_set_continuous    (EvDocumentModel *model,
 						      gboolean         continuous);
 gboolean         ev_document_model_get_continuous    (EvDocumentModel *model);
-void             ev_document_model_set_dual_page     (EvDocumentModel *model,
-						      gboolean         dual_page);
-gboolean         ev_document_model_get_dual_page     (EvDocumentModel *model);
 void             ev_document_model_set_dual_page_odd_pages_left (EvDocumentModel *model,
 								 gboolean         odd_left);
 gboolean         ev_document_model_get_dual_page_odd_pages_left (EvDocumentModel *model);
@@ -85,6 +93,14 @@ void             ev_document_model_set_fullscreen    (EvDocumentModel *model,
 						      gboolean         fullscreen);
 gboolean         ev_document_model_get_fullscreen    (EvDocumentModel *model);
 
+/* deprecated */
+
+EV_DEPRECATED_FOR(ev_document_model_set_page_layout)
+void             ev_document_model_set_dual_page     (EvDocumentModel *model,
+						      gboolean         dual_page);
+EV_DEPRECATED_FOR(ev_document_model_get_page_layout)
+gboolean         ev_document_model_get_dual_page     (EvDocumentModel *model);
+
 G_END_DECLS
 
 #endif /* __EV_DOCUMENT_MODEL_H__ */
