@@ -3,6 +3,11 @@
 #include "ev-job-scheduler.h"
 #include "ev-view-private.h"
 
+typedef enum {
+        SCROLL_DIRECTION_DOWN,
+        SCROLL_DIRECTION_UP
+} ScrollDirection;
+
 typedef struct _CacheJobInfo
 {
 	EvJob *job;
@@ -40,6 +45,7 @@ struct _EvPixbufCache
 	EvDocumentModel *model;
 	int start_page;
 	int end_page;
+        ScrollDirection scroll_direction;
 	gboolean inverted_colors;
 
 	gsize max_size;
@@ -705,6 +711,44 @@ add_job_if_needed (EvPixbufCache *pixbuf_cache,
 		 priority);
 }
 
+static void
+add_prev_jobs_if_needed (EvPixbufCache *pixbuf_cache,
+                         gint           rotation,
+                         gfloat         scale)
+{
+        CacheJobInfo *job_info;
+        int page;
+        int i;
+
+        for (i = pixbuf_cache->preload_cache_size - 1; i >= FIRST_VISIBLE_PREV(pixbuf_cache); i--) {
+                job_info = (pixbuf_cache->prev_job + i);
+                page = pixbuf_cache->start_page - pixbuf_cache->preload_cache_size + i;
+
+                add_job_if_needed (pixbuf_cache, job_info,
+                                   page, rotation, scale,
+                                   EV_JOB_PRIORITY_LOW);
+        }
+}
+
+static void
+add_next_jobs_if_needed (EvPixbufCache *pixbuf_cache,
+                         gint           rotation,
+                         gfloat         scale)
+{
+        CacheJobInfo *job_info;
+        int page;
+        int i;
+
+        for (i = 0; i < VISIBLE_NEXT_LEN(pixbuf_cache); i++) {
+                job_info = (pixbuf_cache->next_job + i);
+                page = pixbuf_cache->end_page + 1 + i;
+
+                add_job_if_needed (pixbuf_cache, job_info,
+                                   page, rotation, scale,
+                                   EV_JOB_PRIORITY_LOW);
+        }
+}
+
 static void
 ev_pixbuf_cache_add_jobs_if_needed (EvPixbufCache *pixbuf_cache,
 				    gint           rotation,
@@ -723,24 +767,33 @@ ev_pixbuf_cache_add_jobs_if_needed (EvPixbufCache *pixbuf_cache,
 				   EV_JOB_PRIORITY_URGENT);
 	}
 
-	for (i = FIRST_VISIBLE_PREV(pixbuf_cache); i < pixbuf_cache->preload_cache_size; i++) {
-		job_info = (pixbuf_cache->prev_job + i);
-		page = pixbuf_cache->start_page - pixbuf_cache->preload_cache_size + i;
+        if (pixbuf_cache->scroll_direction == SCROLL_DIRECTION_UP) {
+                add_prev_jobs_if_needed (pixbuf_cache, rotation, scale);
+                add_next_jobs_if_needed (pixbuf_cache, rotation, scale);
+        } else {
+                add_next_jobs_if_needed (pixbuf_cache, rotation, scale);
+                add_prev_jobs_if_needed (pixbuf_cache, rotation, scale);
+        }
+}
 
-		add_job_if_needed (pixbuf_cache, job_info,
-				   page, rotation, scale,
-				   EV_JOB_PRIORITY_LOW);
-	}
+static ScrollDirection
+ev_pixbuf_cache_get_scroll_direction (EvPixbufCache *pixbuf_cache,
+                                      gint           start_page,
+                                      gint           end_page)
+{
+        if (start_page < pixbuf_cache->start_page)
+                return SCROLL_DIRECTION_UP;
 
-	for (i = 0; i < VISIBLE_NEXT_LEN(pixbuf_cache); i++) {
-		job_info = (pixbuf_cache->next_job + i);
-		page = pixbuf_cache->end_page + 1 + i;
+        if (end_page > pixbuf_cache->end_page)
+                return SCROLL_DIRECTION_DOWN;
 
-		add_job_if_needed (pixbuf_cache, job_info,
-				   page, rotation, scale,
-				   EV_JOB_PRIORITY_LOW);
-	}
+        if (start_page > pixbuf_cache->start_page)
+                return SCROLL_DIRECTION_DOWN;
 
+        if (end_page < pixbuf_cache->end_page)
+                return SCROLL_DIRECTION_UP;
+
+        return pixbuf_cache->scroll_direction;
 }
 
 void
@@ -758,6 +811,8 @@ ev_pixbuf_cache_set_page_range (EvPixbufCache  *pixbuf_cache,
 	g_return_if_fail (end_page >= 0 && end_page < ev_document_get_n_pages (pixbuf_cache->document));
 	g_return_if_fail (end_page >= start_page);
 
+        pixbuf_cache->scroll_direction = ev_pixbuf_cache_get_scroll_direction (pixbuf_cache, start_page, end_page);
+
 	/* First, resize the page_range as needed.  We cull old pages
 	 * mercilessly. */
 	ev_pixbuf_cache_update_range (pixbuf_cache, start_page, end_page, rotation, scale);
@@ -1030,7 +1085,8 @@ ev_pixbuf_cache_get_selection_region (EvPixbufCache *pixbuf_cache,
 	 * assumption that it'll be updated later and we can scale it as need
 	 * be */
 	if (job_info->job && EV_JOB_RENDER (job_info->job)->include_selection)
-		return job_info->selection_region;
+		return job_info->selection_region && !cairo_region_is_empty(job_info->selection_region) ?
+                        job_info->selection_region : NULL;
 
 	/* Now, lets see if we need to resize the region.  If we do, we clear the
 	 * old one. */
@@ -1059,7 +1115,8 @@ ev_pixbuf_cache_get_selection_region (EvPixbufCache *pixbuf_cache,
 		g_object_unref (rc);
 		ev_document_doc_mutex_unlock ();
 	}
-	return job_info->selection_region;
+	return job_info->selection_region && !cairo_region_is_empty(job_info->selection_region) ?
+                job_info->selection_region : NULL;
 }
 
 static void
@@ -1195,12 +1252,12 @@ ev_pixbuf_cache_get_selection_list (EvPixbufCache *pixbuf_cache)
 		}
 
 		if (pixbuf_cache->prev_job[i].selection_points.x1 != -1) {
-			selection = g_new0 (EvViewSelection, 1);
+			selection = g_slice_new0 (EvViewSelection);
 			selection->page = page;
 			selection->rect = pixbuf_cache->prev_job[i].selection_points;
 			if (pixbuf_cache->prev_job[i].selection_region)
 				selection->covered_region = cairo_region_reference (pixbuf_cache->prev_job[i].selection_region);
-			retval = g_list_append (retval, selection);
+			retval = g_list_prepend (retval, selection);
 		}
 		
 		page ++;
@@ -1209,12 +1266,12 @@ ev_pixbuf_cache_get_selection_list (EvPixbufCache *pixbuf_cache)
 	page = pixbuf_cache->start_page;
 	for (i = 0; i < PAGE_CACHE_LEN (pixbuf_cache); i++) {
 		if (pixbuf_cache->job_list[i].selection_points.x1 != -1) {
-			selection = g_new0 (EvViewSelection, 1);
+			selection = g_slice_new0 (EvViewSelection);
 			selection->page = page;
 			selection->rect = pixbuf_cache->job_list[i].selection_points;
 			if (pixbuf_cache->job_list[i].selection_region)
 				selection->covered_region = cairo_region_reference (pixbuf_cache->job_list[i].selection_region);
-			retval = g_list_append (retval, selection);
+			retval = g_list_prepend (retval, selection);
 		}
 		
 		page ++;
@@ -1225,18 +1282,18 @@ ev_pixbuf_cache_get_selection_list (EvPixbufCache *pixbuf_cache)
 			break;
 
 		if (pixbuf_cache->next_job[i].selection_points.x1 != -1) {
-			selection = g_new0 (EvViewSelection, 1);
+			selection = g_slice_new0 (EvViewSelection);
 			selection->page = page;
 			selection->rect = pixbuf_cache->next_job[i].selection_points;
 			if (pixbuf_cache->next_job[i].selection_region)
 				selection->covered_region = cairo_region_reference (pixbuf_cache->next_job[i].selection_region);
-			retval = g_list_append (retval, selection);
+			retval = g_list_prepend (retval, selection);
 		}
 		
 		page ++;
 	}
 
-	return retval;
+	return g_list_reverse (retval);
 }
 
 void
