@@ -61,7 +61,6 @@ typedef struct {
 typedef struct {
 	gboolean in_drag;
 	GdkPoint start;
-	gboolean in_selection;
 	GList *selections;
 	EvSelectionStyle style;
 } SelectionInfo;
@@ -175,6 +174,9 @@ struct _EvView {
 	/* Common for button press handling */
 	int pressed_button;
 
+	/* Key bindings propagation */
+	gboolean key_binding_handled;
+
 	/* Information for middle clicking and dragging around. */
 	DragInfo drag_info;
 	
@@ -212,6 +214,7 @@ struct _EvView {
 	gboolean caret_enabled;
 	gint     cursor_offset;
 	gint     cursor_page;
+	gdouble  cursor_line_offset;
 	gboolean cursor_visible;
 	guint    cursor_blink_timeout_id;
 	guint    cursor_blink_time;
@@ -220,21 +223,25 @@ struct _EvView {
 struct _EvViewClass {
 	GtkContainerClass parent_class;
 
-	void    (*binding_activated)	  (EvView         *view,
-					   GtkScrollType   scroll,
-					   gboolean        horizontal);
-	void    (*handle_link)		  (EvView         *view,
-					   EvLink         *link);
-	void    (*external_link)	  (EvView         *view,
-					   EvLinkAction   *action);
-	void    (*popup_menu)		  (EvView         *view,
-					   GList          *items);
-	void    (*selection_changed)      (EvView         *view);
-	void    (*sync_source)            (EvView         *view,
-					   EvSourceLink   *link);
-	void    (*annot_added)            (EvView         *view,
-					   EvAnnotation   *annot);
-	void    (*layers_changed)         (EvView         *view);
+        void     (*scroll)            (EvView         *view,
+				       GtkScrollType   scroll,
+				       GtkOrientation  orientation);
+        void     (*handle_link)       (EvView         *view,
+				       EvLink         *link);
+        void     (*external_link)     (EvView         *view,
+				       EvLinkAction   *action);
+        void     (*popup_menu)        (EvView         *view,
+				       GList          *items);
+        void     (*selection_changed) (EvView         *view);
+        void     (*sync_source)       (EvView         *view,
+				       EvSourceLink   *link);
+        void     (*annot_added)       (EvView         *view,
+				       EvAnnotation   *annot);
+        void     (*layers_changed)    (EvView         *view);
+        gboolean (*move_cursor)       (EvView         *view,
+				       GtkMovementStep step,
+				       gint            count,
+				       gboolean        extend_selection);
 };
 
 void _get_page_size_for_scale_and_rotation (EvDocument *document,
