@@ -76,6 +76,9 @@ void            ev_view_find_started              (EvView         *view,
 						   EvJobFind      *job);
 void            ev_view_find_next                 (EvView         *view);
 void            ev_view_find_previous             (EvView         *view);
+void            ev_view_find_set_result           (EvView         *view,
+						   gint            page,
+						   gint            result);
 void            ev_view_find_search_changed       (EvView         *view);
 void     	ev_view_find_set_highlight_search (EvView         *view,
 						   gboolean        value);
