@@ -67,7 +67,7 @@ static guint signals[N_SIGNALS];
 
 G_DEFINE_TYPE_EXTENDED (EvSidebarAnnotations,
                         ev_sidebar_annotations,
-                        GTK_TYPE_VBOX,
+                        GTK_TYPE_BOX,
                         0,
                         G_IMPLEMENT_INTERFACE (EV_TYPE_SIDEBAR_PAGE,
 					       ev_sidebar_annotations_page_iface_init))
@@ -232,7 +232,7 @@ ev_sidebar_annotations_init (EvSidebarAnnotations *ev_annots)
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (ev_annots->priv->notebook), FALSE);
 	ev_sidebar_annotations_add_annots_list (ev_annots);
 	ev_sidebar_annotations_add_annots_palette (ev_annots);
-	gtk_container_add (GTK_CONTAINER (ev_annots), ev_annots->priv->notebook);
+        gtk_box_pack_start (GTK_BOX (ev_annots), ev_annots->priv->notebook, TRUE, TRUE, 0);
 	gtk_widget_show (ev_annots->priv->notebook);
 }
 
@@ -300,7 +300,9 @@ ev_sidebar_annotations_class_init (EvSidebarAnnotationsClass *klass)
 GtkWidget *
 ev_sidebar_annotations_new (void)
 {
-	return GTK_WIDGET (g_object_new (EV_TYPE_SIDEBAR_ANNOTATIONS, NULL));
+	return GTK_WIDGET (g_object_new (EV_TYPE_SIDEBAR_ANNOTATIONS,
+                                         "orientation", GTK_ORIENTATION_VERTICAL,
+                                         NULL));
 }
 
 void
