@@ -101,7 +101,7 @@ static void         adjustment_changed_cb                  (EvSidebarThumbnails
 
 G_DEFINE_TYPE_EXTENDED (EvSidebarThumbnails, 
                         ev_sidebar_thumbnails, 
-                        GTK_TYPE_VBOX,
+                        GTK_TYPE_BOX,
                         0, 
                         G_IMPLEMENT_INTERFACE (EV_TYPE_SIDEBAR_PAGE, 
 					       ev_sidebar_thumbnails_page_iface_init))
@@ -294,7 +294,10 @@ ev_sidebar_thumbnails_new (void)
 {
 	GtkWidget *ev_sidebar_thumbnails;
 
-	ev_sidebar_thumbnails = g_object_new (EV_TYPE_SIDEBAR_THUMBNAILS, NULL);
+	ev_sidebar_thumbnails = g_object_new (EV_TYPE_SIDEBAR_THUMBNAILS,
+                                              "orientation", GTK_ORIENTATION_VERTICAL,
+                                              NULL);
+
 
 	return ev_sidebar_thumbnails;
 }
@@ -420,8 +423,6 @@ add_range (EvSidebarThumbnails *sidebar_thumbnails,
 						    page, priv->rotation,
 						    get_scale_for_page (sidebar_thumbnails, page));
                         ev_job_thumbnail_set_has_frame (EV_JOB_THUMBNAIL (job), FALSE);
-			ev_job_scheduler_push_job (EV_JOB (job), EV_JOB_PRIORITY_HIGH);
-			
 			g_object_set_data_full (G_OBJECT (job), "tree_iter",
 						gtk_tree_iter_copy (&iter),
 						(GDestroyNotify) gtk_tree_iter_free);
@@ -431,6 +432,7 @@ add_range (EvSidebarThumbnails *sidebar_thumbnails,
 			gtk_list_store_set (priv->list_store, &iter,
 					    COLUMN_JOB, job,
 					    -1);
+			ev_job_scheduler_push_job (EV_JOB (job), EV_JOB_PRIORITY_HIGH);
 			
 			/* The queue and the list own a ref to the job now */
 			g_object_unref (job);
